/**
 * Copyright 2006 Envoi Solutions LLC
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.codehaus.jettison.mapped;

import java.io.OutputStream;
import org.codehaus.jettison.AbstractDOMDocumentSerializer;

/**
 * JSON Mapped DOM serializer
 * 
 * @author Thomas.Diesler@jboss.com
 * @author <a href="mailto:dejan@nighttale.net">Dejan Bosanac</a>
 * @since 21-Mar-2008
 */
public class MappedDOMDocumentSerializer extends
		AbstractDOMDocumentSerializer {

	public MappedDOMDocumentSerializer(OutputStream output, Configuration con) {
		super(output, new MappedXMLOutputFactory(con));
	}
	
}
